﻿//MysticRose.jsx
//An InDesign ExtendScript by Silicon Publishing, Inc.
//
//Draws a "mystic rose" geometric pattern, a type of construction also
//known as "string art".
//
//Our web site says, "Silicon Publishing, Inc. provides electronic publishing solutions and customizations 
//that automate the distribution of information between multiple sources and destinations." But what does
//that really mean? We make custom software for publishing. Some of it is for intense, data-driven, high-
//volume applications--think: directories, catalogs, and customized itineraries. Some of it is for web-to-print 
//or print-to-web applications--think: business cards, flyers, brochures, data sheets. Some of it is none of the 
//above. Use your imagination--if there's some chance that we can make your work easier, drop us a line! 
//sales@siliconpublishing.com
//
//
app.scriptPreferences.userInteractionLevel = UserInteractionLevels.INTERACT_WITH_ALL;
main();
function main(){
	if(app.documents.length!=0){
		if(app.activeWindow.constructor.name == "LayoutWindow"){
			var array = displayDialog();
			var drawRose = array[0];
			if(drawRose == true){
				var oldXUnits = app.activeDocument.viewPreferences.horizontalMeasurementUnits;
				var oldYUnits = app.activeDocument.viewPreferences.verticalMeasurementUnits;
				app.activeDocument.viewPreferences.horizontalMeasurementUnits = MeasurementUnits.POINTS;
				app.activeDocument.viewPreferences.verticalMeasurementUnits = MeasurementUnits.POINTS;
				var centerPoint = array[1];
				var numberOfPoints = array[2];
				var diameter = array[3];
				var skipLength = array[4];
				var evenOddFill = array[5];
				var pathPoints = calculatePolygon(centerPoint, diameter/2, numberOfPoints);
				drawMysticRose(pathPoints, skipLength, evenOddFill, numberOfPoints);
				app.activeDocument.viewPreferences.horizontalMeasurementUnits = oldXUnits;
				app.activeDocument.viewPreferences.verticalMeasurementUnits = oldYUnits;
			}

		}
	}
}
function displayDialog(){
	var labelWidth = 90;
	var dialog = app.dialogs.add({name:"MysticRose"});
	with(dialog.dialogColumns.add()){
		with(dialogRows.add()){
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"Center Point:", minWidth:labelWidth});
			}
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"X:"});
			}
			with(dialogColumns.add()){
				var xField = measurementEditboxes.add({editValue:0, editUnits:MeasurementUnits.POINTS});
			}
		}
		with(dialogRows.add()){
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"", minWidth:labelWidth});
			}
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"Y:"});
			}
			with(dialogColumns.add()){
				var yField = measurementEditboxes.add({editValue:0, editUnits:MeasurementUnits.POINTS});
			}
		}
		labelWidth = 115;
		with(dialogRows.add()){
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"Diameter:", minWidth:labelWidth});
			}
			with(dialogColumns.add()){
				var diameterField = measurementEditboxes.add({editValue:144, editUnits:MeasurementUnits.POINTS});
			}
		}
		with(dialogRows.add()){
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"Number of Points:", minWidth:labelWidth});
			}
			with(dialogColumns.add()){
				var numberOfPointsField = integerEditboxes.add({editValue:21});
			}
		}
		with(dialogRows.add()){
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"Skip Array:", minWidth:labelWidth});
			}
			with(dialogColumns.add()){
				var skipLengthField = textEditboxes.add({editContents:"1,2,3,4,5,6,7,8,9,10"});
			}
		}
		var evenOddCheckbox = checkboxControls.add({staticLabel:"Even/Odd Fill"});
	}
	var result = dialog.show();
	//Turn the skip pattern into the string version of an array.
	var skipLength = "[" + skipLengthField.editContents + "]";
	var numberOfPoints = numberOfPointsField.editValue;
	var diameter = diameterField.editValue;
	var x = xField.editValue;
	var y = yField.editValue;
	var evenOddFill = evenOddCheckbox.checkedState;
	if(result == true){
		drawRose = true;
		dialog.destroy();
	}
	else{
		dialog.destroy();
		drawRose = false;
	}
	return [drawRose, [x, y], numberOfPoints, diameter, skipLength, evenOddFill];
}
function drawMysticRose(pathPoints, skipArray, evenOddFill, numberOfPoints){
	var index;
	var newPathPoints = new Array;
	var done = false;
	var counter = 1;
	var index = 0;	
	skipArray = eval(skipArray);
	newPathPoints.push(pathPoints[0]);
	do{
		for(var skipCounter = 0; skipCounter < skipArray.length;skipCounter++){
			var index = (index + skipArray[skipCounter])%pathPoints.length;
			newPathPoints.push(pathPoints[index]);
		}
		if(index == 0){
            //When we reach the starting point at the end of the sequence
            //of steps, we're done. Note that we can reach the starting point
            //many times *inside* the step sequence.
			done = true;
		}
		else{
			counter ++;
		}
	}while (done != true);
	var polygon = app.activeWindow.activePage.polygons.add();
	polygon.paths.item(0).entirePath = newPathPoints;
	if(evenOddFill == 1){
		var newPolygon = polygon.duplicate();
		polygon.addPath(newPolygon);
	}
}
//Function calculates the points for an n-sided polygon around a given center point.
function calculatePolygon(centerPoint, radius, numberOfPoints){
	angleIncrement = (360/numberOfPoints)*(Math.PI/180);
	var pathPoints = new Array;
	for (pointCounter = 0; pointCounter < numberOfPoints;pointCounter ++){
		x = centerPoint[0] + (radius * Math.cos(angleIncrement*pointCounter));
		y = centerPoint[1] + (radius * Math.sin(angleIncrement*pointCounter));
		pathPoints.push([x, y]);
	}
	return pathPoints;	
}